#    Diagnostic and test task for the  Lyrtech SM (Signal Master)

set frame  SM
set title "Lyrtech SM Client"
set icon   SM

set started 0

set inputcolour  green4
set outputcolour red
set insertcolour blue
set bkgcolour    azure

set LogFlag 1

set verbose 0

set SMServerList {localhost nndhcp183}
set SMServer "select a server"
set SMHost ""

set SMResourceList {}
set SMResource ""

set fpgadirectory [file join $env(MIDASBASE) LyrTech bit]
#set fpga1ffile "undefined"
#set fpga2ffile "undefined"
set fpga1ffile "c:/MIDAS/LyrTech/bit/smquadc64x_fpga1_virtex2_ff1152_ioring.bit"
set fpga2ffile "c:/MIDAS/LyrTech/bit/smquadc64x_fpga2_virtex2_ff1152_ioring.bit"



set dspdirectory [file join $env(MIDASBASE) LyrTech bin]
#set dsp0ffile "undefined"
#set dsp2ffile "undefined"
set dsp0ffile "c:/MIDAS/LyrTech/bin/p1d1_BA50.out"
set dsp2ffile "c:/MIDAS/LyrTech/bin/p1d3_BA50.out"
set dspffmt 3


set dspramaddr  0
set dspramwords 16
set dspraminc 0

set dsprambase 0x90000000
set dsprambasechoice [list \
     "DSP Internal RAM - 0x00000000"  \
     "DSP External Memory interface CE0 - 0x80000000"  \
     "DSP External Memory interface CE1 - 0x90000000"  \
     "DSP External Memory interface CE2 - 0xA0000000"  \
     "DSP External Memory interface CE3 - 0xB0000000"  \
]
set dsprambaseindex 2

set boardID -1
set resourceID =1
set maxboards -1
set boardlist "none"



proc bgerror err {
    global errorInfo tkerrorInfo tkskiperror
    upvar #0 MIDAS_SOAPFaultCode FaultCode
    upvar #0 MIDAS_SOAPFaultString FaultString
    upvar #0 MIDAS_SOAPErrorCode ErrorCode
    upvar #0 MIDAS_SOAPErrorInfo ErrorInfo

    if {$tkskiperror == 1} then {set tkskiperror 0; return}
    set tkerrorInfo $errorInfo

    if {[grab current .] != ""} then {grab release [grab current .]}

    catch {:modify [self frame-name] :busy T}
    if {![info exists FaultCode]} {
        set z [catch {midas_dialog [self frame-name].tkerrorDialog "Error Detected in Tcl Script" \
                "Task '[Get_My_Name]' detected error in Tcl script\n\n$err" "error" 0 "continue" "Stack Trace"} button]
    } else {
        set z [catch {midas_dialog [self frame-name].tkerrorDialog "Error Detected in Tcl Script" \
                "Task '[Get_My_Name]' detected error in Tcl script\n\n$FaultCode\n$FaultString\n$ErrorCode\n$ErrorInfo" \
                       "error" 0 "continue"} button]
    }
    catch {:modify [self frame-name] :busy F}

    if {$z != 0} then {
       puts stderr "midas_dialog: [self frame-name].tkerrorDialog: $button"
       destroy [self frame-name].tkerrorDialog
       return
    }

    if {$button != 0} then {tkStackTrace}
}


proc make-frame {} {
   global env SMServerList SMResourceList

set rows 12
set cols 60

  :frame [self frame-name] :label "[self frame-title]" \
     :icon-name "[self icon-title]"

  :panel P0 :layout V :padx 5 :pady 5
    :layout H; :col-gap 10; :row-gap 10

    :text WEBSERVERT         :label "SM WebServer" :w 20  :value "select a server"
    :choice-stack WEBSERVERC :label "" :strings $SMServerList
    :hskip 30
    :checkbox VERBOSE        :label "Enable Verbose Option" 

    :next-row T
    :number MODULE-NUM :label "SM Board Selection:" :min -1 :max -1 :w 5 :value -1
    :menubutton MODULE-SEL :label "" :menu "none"
    :hskip 50
    :button INFO :label "Report Board Information"  :command click-boardinfo

    :next-row T
    :choice-stack RESOURCES :label "Board Resources" :strings $SMResourceList

  :panel P1 :layout V :padx 5 :pady 5
    :galley LOG :rows $rows :cols $cols :font fixed
     configuretags

  :panel P2 :layout V :padx 5 :pady 5
    :layout H; :col-gap 20; :row-gap 10

  :panel P3 :layout V :padx 20 :pady 5
    :layout H; :col-gap 20; :row-gap 10

    :button CLEAR   :label "Clear log window"     :w 20 :command ":modify LOG :empty T"
    :button REPORT0  :label "Show fpga1 registers" :w 20 :command "click-boardregisters 0"
    :button REPORT2  :label "Show fpga2 registers" :w 20 :command "click-boardregisters 2"
#    :button SAVE    :label "Save Settings"        :command click-save
#    :button RESTORE :label "Restore Settings"     :command click-restore

    :next-row T
    :button P50 :label "DSP Memory"            :w 20 :command do_dspRAM
    :button P51 :label "FPGA1 Programming"      :w 20 :command do_fpga1_programming
    :button P52 :label "FPGA2 Programming"       :w 20 :command do_fpga2_programming


   open-frame

  :galley_size LOG $rows $cols

   :show T
}

proc do_fpga1_programming {} {
    upvar #0 fpga1ffile f1filename
    upvar #0 dsp0ffile d0filename
    upvar #0  dspffmt format

  :frame "[self frame-name].fpga1" :label "FPGA1 programming" :resize F :transient T

   :panel F1 :layout T :padx 5 :pady 5
     :layout H; :row-gap 10; :col-gap 10

      :text   FPGA1FILE    :label "Bitstream file for FPGA" :w 60 :value $f1filename
      :button FPGA1BROWSE :label "Browse"

      :next-row T
      :col-gap 20
      :button FPGA1DONE      :label "Done" :command "frame-quit [self frame-name].fpga"
      :button FPGA1UPLOAD    :label "Upload FPGA bitstream file"  :command "do_UpLoadfpga 1"
      :button FPGA1PGM       :label "Program FPGA"                :command "do_Programfpga 1"
      :button FPGA1UPLOADPGM :label "Upload & Program FPGA"       :command "do_UpLoadProgramfpga 1"

   :panel D0 :layout T :padx 5 :pady 5
     :layout H; :row-gap 10; :col-gap 10

      :text   DSP0FILE      :label "Program file for DSP0" :valuex 20c :w 60 :value $d0filename
      :button DSP0BROWSE  :label "Browse"

      :next-row T
      :choice-stack DSPFMT :label "File format for DSP" :valuex 20c :value $format \
          :strings [list "undefined" "Motorola S records" "Analog Devices Host Port Loadable" "binary"]

      :next-row T
      :col-gap 20
      :button DSP0DONE      :label "Done" :command "frame-quit [self frame-name].dsp"
      :button DSP0UPLOAD    :label "Upload DSP program file"    :command "do_UpLoaddsp 1 0"
      :button DSP0PGM       :label "Program DSP"                :command "do_Programdsp 1 0"
      :button DSP0UPLOADPGM :label "Upload & Program DSP"       :command "do_UpLoadProgramdsp 1 0"

  :show T
}
proc do_fpga2_programming {} {
    upvar #0 fpga2ffile f2filename
    upvar #0 dsp2ffile d2filename
    upvar #0  dspffmt format

  :frame "[self frame-name].fpga2" :label "FPGA2 programming" :resize F :transient T

   :panel F2 :layout T :padx 5 :pady 5
     :layout H; :row-gap 10; :col-gap 10

      :text   FPGA2FILE    :label "Bitstream file for FPGA" :w 60 :value $f2filename
      :button FPGA2BROWSE :label "Browse"

      :next-row T
      :col-gap 20
      :button FPGA2DONE      :label "Done" :command "frame-quit [self frame-name].fpga"
      :button FPGA2UPLOAD    :label "Upload FPGA bitstream file"  :command "do_UpLoadfpga 2"
      :button FPGA2PGM       :label "Program FPGA"                :command "do_Programfpga 2"
      :button FPGA2UPLOADPGM :label "Upload & Program FPGA"       :command "do_UpLoadProgramfpga 2"

   :panel D2 :layout T :padx 5 :pady 5
     :layout H; :row-gap 10; :col-gap 10

      :text   DSP2FILE      :label "Program file for DSP2" :valuex 20c :w 60 :value $d2filename
      :button DSP2BROWSE  :label "Browse"

      :next-row T
      :choice-stack DSPFMT :label "File format for DSP" :valuex 20c :value $format \
          :strings [list "undefined" "Motorola S records" "Analog Devices Host Port Loadable" "binary"]

      :next-row T
      :col-gap 20
      :button DSP2DONE      :label "Done" :command "frame-quit [self frame-name].dsp"
      :button DSP2UPLOAD    :label "Upload DSP program file"    :command "do_UpLoaddsp 2 2"
      :button DSP2PGM       :label "Program DSP"                :command "do_Programdsp 2 2"
      :button DSP2UPLOADPGM :label "Upload & Program DSP"       :command "do_UpLoadProgramdsp 2 2"

  :show T
}

proc do_dspRAM {} {
   global dsprambase dspramaddr dspramwords dsprambasechoice dspraminc dsprambaseindex

  :frame [self frame-name].dspram :label "Display DSP Memory" :resize F :transient T

    :panel DSPRAM :layout T :padx 10 :pady 10
     :layout H; :col-gap 10; :row-gap 10
     :choice-stack DSPRAMBASEC  :label "Area base address" :strings $dsprambasechoice :value $dsprambaseindex :valuex 24c
     :text DSPRAMBASET :label "" :value $dsprambase :w 12
     :next-row T
     :text DSPRAMADDR      :label "Address Offset"  :value $dspramaddr   :valuex 24c  :w 12
     :button DSPRAMADDRZ   :label "Reset to zero"
     :hskip 30
     :checkbox DSPRAMADDRI :label "Enable Address Increment" :value $dspraminc
     :next-row T
     :text DSPRAMWORDS   :label "Words to read"   :value $dspramwords  :valuex 24c  :w 12
     :hskip 25
     :text DSPRAMDATA    :label "Data to write"   :w 50
     :next-row T
     :layout V
     :message DSPRAMM1 :label "Memory Contents" :bold T
     :galley DSPRAMINFO :rows 20 :cols 110 :deaf T :font fixed

  :panel P3 :layout V :padx 5 :pady 5
    :layout H; :col-gap 20

       :button DSPRAMDONE  :label "Done" :command "frame-quit [self frame-name].dspram"
       :button DSPRAMCLR   :label "Clear Window" :command ":modify DSPRAMINFO :append F"
       :button DSPRAMREAD  :label "Read Memory"  :command  do_dspramdisplay
       :button DSPRAMWRITE :label "Write Memory"  :command  do_dspramwrite
  
   :show T
}

proc click-fpga1file {args} {
    upvar #0 fpgadirectory  directory
    upvar #0 fpga1ffile f1filename

    set f1filename [first $args]
    set directory [file dirname $f1filename]
}
proc click-fpga2file {args} {
    upvar #0 fpgadirectory  directory
    upvar #0 fpga2ffile f2filename

    set f2filename [first $args]
    set directory [file dirname $f2filename]
}


proc click-fpga1browse {args} {
    global env
    upvar #0 fpgadirectory  directory
    upvar #0 fpga1ffile f1filename

    	option add *Canvas.background  #c0c020
	set types {
	{ {Bitstream Files} {*.bit} }
	{ {All Files } {*} }
	}

	set f [tk_getOpenFile -parent [self frame-name] \
		  -title "FPGA bitstream File Selection"  \
		  -initialdir $directory -filetypes $types]
	option add *Canvas.background  $env(tkcanvasbackground)
	if {$f == ""} then {return}

	set f1filename $f   ;#  remember current choice
      set directory [file dirname $f]

      :modify FPGA1FILE :value $f1filename
}
proc click-fpga2browse {args} {
    global env
    upvar #0 fpgadirectory  directory
    upvar #0 fpga2ffile f2filename

    	option add *Canvas.background  #c0c020
	set types {
	{ {Bitstream Files} {*.bit} }
	{ {All Files } {*} }
	}

	set f [tk_getOpenFile -parent [self frame-name] \
		  -title "FPGA bitstream File Selection"  \
		  -initialdir $directory -filetypes $types]
	option add *Canvas.background  $env(tkcanvasbackground)
	if {$f == ""} then {return}

	set f2filename $f   ;#  remember current choice
      set directory [file dirname $f]

      :modify FPGA2FILE :value $f2filename
}


proc do_UpLoadfpga {id} {
    upvar #0 fpga[set id]ffile file

    set-footer "[self frame-name].fpga[set id]"  "Uploading FPGA bitstream file.... This takes about 10 secs"
    set z [catch {SMClient__SMUpLoad $file} rc]
    clear-footer "[self frame-name].fpga[set id]" 

    if {$z == 0} {
         set-footer "[self frame-name].fpga[set id]"   "Uploaded file $file successful"
     } else {
         bgerror "Uploading file $file failed\n$rc"
    }
}

proc do_Programfpga {id} {
    global boardID resourceID SMResourceList
    upvar #0 fpga[set id]ffile file
	
    if {$id == 1} {set resourceID 0} {set resourceID 2}

    set-footer "[self frame-name].fpga[set id]"  "loading FPGA.... This takes about 10 secs"
    set z [catch {SMClient__SMSetFPGALoad [expr ($resourceID << 8) + $boardID] $file} rc]
    clear-footer "[self frame-name].fpga[set id]" 

    if {$rc == "NONE"} {global MIDAS_SOAPErrorInfo; set rc $MIDAS_SOAPErrorInfo}

    if {$z == 0} {
        if {$rc == 0} then {
             set ID [lindex $SMResourceList $resourceID]
             set-footer "[self frame-name].fpga[set id]"   "FpgaLoad for board $boardID $ID file $file successful"
        } else {
             midas-warning "FpgaLoad for board $boardID file $file failed\n$rc"
        }
    } else {
         midas-warning "FpgaLoad for board $boardID file $file failed\n$rc"
    }
}

proc do_UpLoadProgramfpga {id}  {do_UpLoadfpga $id; do_Programfpga $id}

proc click-dsp0file {args} {
    upvar #0 dspdirectory  directory
    upvar #0 dsp0ffile d0filename

    set d0filename [first $args]
    set directory [file dirname $d0filename]
}
proc click-dspfile {args} {
    upvar #0 dspdirectory  directory
    upvar #0 dsp2ffile d2filename

    set d2filename [first $args]
    set directory [file dirname $d2filename]
}


proc click-dspfmt {vl} {
    upvar #0 dspffmt format

    set format $vl
}

proc click-dsp0browse {args} {
    global env
    upvar #0 dspdirectory  directory
    upvar #0 dsp0ffile d0filename

    	option add *Canvas.background  #c0c020
	set types {
	{ {Bitstream Files} {*.out} }
	{ {All Files } {*} }
	}

	set f [tk_getOpenFile -parent [self frame-name] \
		  -title "FPGA bitstream File Selection"  \
		  -initialdir $directory -filetypes $types]
	option add *Canvas.background  $env(tkcanvasbackground)
	if {$f == ""} then {return}

      set d0filename $f   ;#  remember current choice
      set directory [file dirname $f]

      :modify DSP0FILE :value $d0filename
}
proc click-dsp2browse {args} {
    global env
    upvar #0 dspdirectory  directory
    upvar #0 dsp2ffile d2filename

    	option add *Canvas.background  #c0c020
	set types {
	{ {Bitstream Files} {*.out} }
	{ {All Files } {*} }
	}

	set f [tk_getOpenFile -parent [self frame-name] \
		  -title "FPGA bitstream File Selection"  \
		  -initialdir $directory -filetypes $types]
	option add *Canvas.background  $env(tkcanvasbackground)
	if {$f == ""} then {return}

      set d2filename $f   ;#  remember current choice
      set directory [file dirname $f]

      :modify DSP2FILE :value $d2filename
}


proc do_UpLoaddsp {f id} {
    upvar #0 dsp[set id]ffile file

    set-footer "[self frame-name].fpga[set f]"  "Uploading DSP program file.... This takes about 10 secs"
    set z [catch {SMClient__SMUpLoad $file} rc]
    clear-footer "[self frame-name].fpga[set f]" 

    if {$z == 0} {
         set-footer "[self frame-name].fpga[set f]"   "Uploaded file $file successful"
     } else {
         bgerror "Uploading file $file failed\n$rc"
    }
}

proc do_Programdsp {f id} {
    global boardID resourceID SMResourceList
    upvar #0 dsp[set id]ffile file
    upvar #0 dspffmt format

    set resourceID $id

    set-footer "[self frame-name].fpga[set f]"  "programming DSP.... This takes a few secs"
    set z [catch {SMClient__SMSetDSPLoad [expr ($resourceID << 8) + $boardID] $resourceID $format $file} rc]
    clear-footer "[self frame-name].fpga[set f]" 

    if {$rc == "NONE"} {global MIDAS_SOAPErrorInfo; set rc $MIDAS_SOAPErrorInfo}

    if {$z == 0} {
        if {$rc == 0} then {
             set ID [lindex $SMResourceList $resourceID]
             set-footer "[self frame-name].fpga[set f]"   "DspLoad for board $boardID $ID file $file successful"
        } else {
             midas-warning "DspLoad for board $boardID file $file failed\n$rc"
        }
    } else {
         midas-warning "DspLoad for board $boardID file $file failed\n$rc"
    }
}

proc do_UpLoadProgramdsp {f id}  {do_UpLoaddsp $f $id; do_Programdsp $f $id}



proc redisplay {} {
    catch {update_}
}

proc click-configure {W} {:galley_resize LOG}

proc configuretags {} {
   global inputcolour outputcolour insertcolour bkgcolour

    :modify LOG :createtag "taginput  -foreground $inputcolour  -background $bkgcolour"
    :modify LOG :createtag "tagoutput -foreground $outputcolour -background $bkgcolour"
    :modify LOG :createtag "taginsert -foreground $insertcolour -background $bkgcolour"
}

proc insert {s} {:modify LOG :usingtag taginsert :strings $s\n; update idletasks}


proc start args {
   global started

   if {$started == 0} then {make-frame} else {open-frame}
   set started 1
   
}

proc click arg {
   global verbose SMHost

   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   clear-footer

   switch -glob $w  {

      WEBSERVERT    {click-server $v}
      WEBSERVERC    {click-server $i}

      MODULE-SEL    -
      MODULE-NUM    {click-select-module $v}

      RESOURCES     {click-select-resource $v}

      VERBOSE       {set verbose $v; if {$SMHost != ""} then {SMClient__SMVerbose $verbose}}
 
      default {
         set p "click-[lowercase $w]"
         if {[info procs $p] == "$p"} then {$p $v} else {set-footer "no handler for widget $w"}
      }
   }

    redisplay 
}

proc click-server {v} {
    global SMServerList SMServer SMResourceList SMResource
    global boardID resourceID maxboards boardlist
    global verbose

    global SMHost SMPort

    set SMServer $v
    :modify WEBSERVERT :value $SMServer
    :modify WEBSERVERC :value [lsearch $SMServerList $SMServer]

    set SMHost $SMServer
    package require SMClient

    SMClient__SMVerbose $verbose
    set maxboards [SMClient__SMInit]
    
    set boardlist {}
    for {set i 0} {$i < $maxboards} {incr i} {lappend boardlist $i}
    set boardID 0
    :modify MODULE-SEL :menu $boardlist
    :modify MODULE-NUM :min 0 :max [expr $maxboards-1]
    :modify MODULE-NUM :value $boardID

    insert "Initialised SM Web Service at $SMServer: $maxboards SM boards discovered"

    set SMResourceList [split [SMClient__SMGetUsers $boardID] |]
    set SMResource [lindex $SMResourceList 0]
    set resourceID 0
    :modify RESOURCES :menu $SMResourceList
    :modify RESOURCES :value 0
}

proc click-select-module {vl} {
    global boardID maxboards

    set boardID $vl
    if {$boardID < 0} {set boardID 0}
    if {$boardID >= $maxboards} {set boardID [expr $maxboards-1]}

    :modify MODULE-NUM :value $boardID
}

proc click-select-resource {i} {
    global resourceID

    set resourceID $i
}

proc update_ {} {
    global boardID

}

proc click-boardinfo {args} {
    global boardID

   set type [SMClient__SMGetSMType $boardID]
   switch $type {
       0x403   {insert "Board Type is QuadC6xx-based SignalMaster"}
       default {insert "Unexpected Board Type ($type)"}
   }
   
   foreach ID {1 2 0x100 0x200 0x201 0x400 0x401} {
       set version [SMClient__SMGetVersion $boardID $ID]
       set desc [lreplace $version 0 2]
       if {$desc != "n/a"} {insert "$desc version [expr [first $version]].[expr [second $version]].[expr [third $version]]"}
   }
}

proc click-boardregisters {args} {
   global boardID resourceID
   global DataRAM

   set resID $args
   if {$resID < 2} {set fpga 1} {set fpga 2}
    
   if {$resID > 3} {midas-warning "You cannot only do this for a DSP resource"; return}

   set-footer "Reading data from DSP RAM"

   set baseaddress 0x90000000
   set length 1
   set DataRAM [SMClient__SMGetDSPRead [expr ($resID << 8) + $boardID] $baseaddress $length]

   insert "FPGA$fpga version [format 0x%08x [first $DataRAM]]"

   set baseaddress 0x90000040
   set length 16
   set DataRAM [SMClient__SMGetDSPRead [expr ($resID << 8) + $boardID] $baseaddress $length]

   set info ""
   foreach word $DataRAM {
       append info "[format 0x%08x $word]  "
   }
   
   insert "User control registers: $info"

   clear-footer
}

proc  do_dspramdisplay {} {
   global boardID resourceID
   global  dsprambase dspramaddr dspramwords dspraminc
   global DataRAM

   if {$resourceID > 3} {midas-warning "You cannot only do this for a DSP resource"; return}

   set-footer [self frame-name].dspram "Reading data from DSP RAM"

   set baseaddress [expr $dsprambase + $dspramaddr]
   set length [expr $dspramwords]
   set DataRAM [SMClient__SMGetDSPRead [expr ($resourceID << 8) + $boardID] $baseaddress $length]

   set info ""
   set i 0
   set offset 0
   foreach word $DataRAM {
      if {$i == 0} {append info "[format "0x%08x" [expr $baseaddress + $offset]]: "}
      append info "[format 0x%08x $word]  "
      incr i
      if {$i == 8} then {append info "\n"; set i 0}
      incr offset 4
   }
   :modify DSPRAMINFO :strings $info\n

   clear-footer [self frame-name].dspram

   if {$dspraminc} {click-dspramaddr [expr $dspramaddr + ($dspramwords*4)]}
}

proc click-dsprambasec {v} {
   global  dsprambase dsprambasechoice

   set dsprambase  [last [lindex $dsprambasechoice $v]]
   :modify DSPRAMBASET :value [format "0x%08x" $dsprambase]
}

proc click-dsprambaset {v} {
   global  dsprambase
   if {[scan-int $v] == "???"} then {return}
   set dsprambase $v
   :modify DSPRAMBASET :value [format "0x%08x" $dsprambase]
   :modify DSPRAMBASEC :value -1
}

proc click-dspramaddr {v} {
   global dspramaddr
   if {[scan-int $v] == "???"} then {return}
   set dspramaddr $v
   :modify DSPRAMADDR :value [format "0x%08x" $dspramaddr]
}

proc click-dspramaddrz {args} {click-dspramaddr 0}

proc click-dspramwords {v} {
   global dspramwords
   if {[scan-int $v] == $v && $v >= 0 && $v <= 0x400} then {set dspramwords $v}
   :modify DSPRAMWORDS :value [format "0x%02x" $dspramwords]
}

proc click-dspramaddri {v} {
    global dspraminc
    set dspraminc $v
}

proc click-dspramdata {v} {
    global dspramdata
    set dspramdata $v
}

proc do_dspramwrite {} {
    global boardID resourceID
    global  dsprambase dspramaddr dspramwords dspraminc dspramdata
    global DataRAM

#   if {$resourceID > 3} {midas-warning "You cannot only do this for a DSP resource"; return}

    set baseaddress [expr $dsprambase + $dspramaddr]
    set length [llength $dspramdata]
    set DataRAM $dspramdata

    SMClient__SMSetDSPWrite [expr ($resourceID << 8) + $boardID] $baseaddress $length $DataRAM

}

