


set CustomGUI 0
set Customcfgfile ""

set WriteIF 0x70     ;#   User Register 0
set ReadIF 0x74      ;#   User Register 1

set clock_frequency      105.0

set decimation_factor    4
set shaping_time         6.0
set trapezoid_width      4.4
set peak_separation      8.5
set decay_time_constant  50.0
set peak_sample_time     7.2
set baseline_update_time 2
set baseline_average     4
set baseline_threshold   16384
set mode                 0

set reg 0     ;# control register
set MWDr$reg 0
for {set reg 16} {$reg <= 31} {incr reg} {set MWDr$reg 0}
for {set i 0} {$i < 11} {incr i} {set MWDp$i 0}

set UserReg {}

set Reg0x20 0     ;#  CFD Threshold
set Reg0x31 0     ;#  PreTrigger Position
set Reg0x3a 0
set Reg0x3b 0
set Reg0x3c 0
set Reg0x3d 0


set ControlWordMenu [list \
        "Disable Channel" "Disable Baseline subtraction" "Data Shift (LSB)" "Data Shift (MSB)" "Trigger Polarity: 0 = +ve, 1 = -ve" \
        "Test Select bit 0" "Test Select bit 1" "Test Select bit 2"   \
        "Data Select (LSB)" "Data Select (MSB)" GPIO_OE "Swap (=1 inverts order of DAC data)" \
        "Trigger: 0=external, 1=internal (CFD)" "Reset tx_rtdex logic" \
        "Enable PileUp tag on MSB of Energy word" "Enable OverRange tag on MSB of BaseLine word" \
]

set StatusWordMenu  [list 0 1 2 3 4 5 6 7 8 9 10 11 12]

set Menu1 {}
for {set i 0} {$i <= 15} {incr i} {lappend Menu1 $i}

set Menu2 {}
for {set i 0x00} {$i < 0x8f} {incr i} {lappend Menu2 [format 0x%02x $i]}


proc VHSADCClient__ReadUserReg {chan reg} {
    global ReadIF boardID

        set regvl [expr ($chan << 28) + ($reg << 16)]        ;#    build data for register
#insert-debug "Writing [format 0x%08x $regvl] to offset $ReadIF"
        VHSADCClient__Vhsadac16WriteFpgaReg $boardID $ReadIF $regvl        ;#    write to user register
        VHSADCClient__Vhsadac16WriteFpgaReg $boardID $ReadIF $regvl        ;#    write to user register
        set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID $ReadIF]  ;# user register
#insert-debug "Read [format 0x%04x $rc] from offset $ReadIF"

    return $rc
}

proc VHSADCClient__WriteUserReg {chan reg vl} {
    global WriteIF boardID

        set regvl [expr ($chan << 28) + ($reg << 16) + $vl]        ;#    build data for register
#insert-debug "Writing [format 0x%08x $regvl] to offset $WriteIF"
        VHSADCClient__Vhsadac16WriteFpgaReg $boardID $WriteIF $regvl        ;#    write to user register
}


proc do_pause  {} {inform sys "click-special 23"}
proc do_resume {} {inform sys "click-special 24"}


proc click-select-chan {vl} {
    global chan
    global NumChan boardID

    if {$vl >= $NumChan($boardID)} {
         midas-warning "Channel not supported by FPGA firmware"
         set chan 0
    } else {
         set chan $vl
    }

    :modify CHAN-NUM :value $chan

    update_UserGUI
}


set user10 0
set user11 0
set user12 0
set user20 0
set user21 0
set user22 0

for {set i 0} {$i <= 31} {incr i} {set RegMask($i) 0x0000ffff}

if {$UserGUIVersion == 2} {     ;#   MWD firmware
  set RegMask(16) 255
  set RegMask(17) 255
  set RegMask(20) 255
  set RegMask(21) 255
  set RegMask(24) 255
  set RegMask(25) 31
  set RegMask(26) 3
}


proc make_UserGUI {} {
    global UserGUIVersion
    global user10 user11 user12
    global user20 user21 user22
    global numchans chan chanlist allchans allmodules
    global ControlWordMenu StatusWordMenu
    global Menu1 Menu2

   global Options Customcfgfile
   global UserReg

#    GUI interface for MWD firmware
 
  :frame "[self frame-name].user" :label "Firmware Parameter Control" :resize F :transient T

   :panel MODULE  :layout T :padx 10 :pady 10
     :layout H; :row-gap 10; :col-gap 10

     :text USER20   :label "VHS-ADC Board Selection" :inactive T :w 3

     :number CHAN-NUM  :label "Channel number:" :min 0 :max [expr $numchans-1] :w 5 :value $chan
     :menubutton CHAN-SEL :label "select:" :strings $chanlist
     :hskip 10
     :checkbox ALL-MODULES :label "Act on ALL modules?"  :value $allmodules
     :checkbox ALL-CHANS   :label "Act on ALL channels?" :value $allchans

   :container C0 :layout T

   :panel P0L :layout L :padx 10 :pady 0
     :layout V; :row-gap 10; :col-gap 10

      :choice USER2CW :label "Control Word" :exclusive F :valuex 15c :layout V :w 40 :strings $ControlWordMenu

   :panel P0M :layout L :padx 0 :pady 0
     :layout V; :row-gap 10; :col-gap 10

      :choice USER2SW :label "Status Word" :exclusive F :layout V :w 3 :inactive T :strings $StatusWordMenu

      :choice XXX :label "legend" :layout V :exclusive F :w 14 \
          :strings [list "0/off/false" "1/on/true"] :value 2 :inactive T

   :panel P0R :layout R :padx 0 :pady 0
     :layout V; :row-gap 10; :col-gap 10

if {$UserGUIVersion == 2} {     ;#   MWD firmware

      :text CLOCK_FREQUENCY      :label "Clock Frequency"  :w 8 :valuex 24c
      :text SHAPING_TIME         :label "Shaping (rise time + flat top)"  :w 8 :valuex 24c
      :text TRAPEZOID_WIDTH      :label "Rise Time"  :w 8 :valuex 24c
      :text PEAK_SAMPLE_TIME     :label "Peak Sample"  :w 8 :valuex 24c
      :text PEAK_SEPARATION      :label "Peak Separation"  :w 8 :valuex 24c
      :text BASELINE_AVERAGE     :label "Baseline Average"  :w 8 :valuex 24c
      :text BASELINE_UPDATE_TIME :label "Baseline Update Clocks"  :w 8 :valuex 24c
      :text BASELINE_THRESHOLD   :label "Baseline Threshold"  :w 8 :valuex 24c
      :text DECAY_TIME_CONSTANT  :label "Decay Time Constant"  :w 8 :valuex 24c
      :text DECIMATION_FACTOR    :label "Decimation"  :w 8 :valuex 24c
      :text MODE                 :label "Mode"  :w 8 :valuex 24c
}

if {$UserGUIVersion == 1} {    

    set Customcfgfile $Options(FpgaRegisters)

    set z [catch {open $Customcfgfile r} f]
    if {$z != 0} then {
          midas-warning "Cannot read file $Customcfgfile\n$f"
          return
    }
    set Usercfg [split [read $f] \n]
    close $f

#    process configuration file

    set UserReg {}

    foreach item $Usercfg {
       if {[string range $item 0 0] == "#"} {continue}
       if {[first $item] == "name"} {continue}
       switch [first $item] {
           choice  {
               set ID [second [split [third $item] =]]
               lappend UserReg $ID
               set options [lreplace $item 0 2]
               set menu {}
               foreach element $options {
                   lappend menu [second [split $element :]]
               }
               :choice MWDR$ID :label "[second $item]" :strings $menu  :valuex 15c
               :next-row T
           }
           slider {
               set ID [second [split [third $item] =]]
               lappend UserReg $ID
               set Units [second [split [lindex $item 3] =]]
               set Range [second [split [lindex $item 4] =]]
               set min [first [split $Range ,]]
               set max [second [split $Range ,]]
               :slider MWDR$ID :label "[second $item]" :min [second [split $min :]] :max [second [split $max :]]  :valuex 15c
               :next-row T
           }
           value {
               set ID [second [split [third $item] =]]
               lappend UserReg $ID
               :text MWDR$ID :label "[second $item]" :w 12 :valuex 15c
               :next-row T
           }
           default {}
       }
    }
}

      :vskip 10
      :text USER2CFD             :label "CFD Threshold (channel)"  :w 8 :valuex 24c
      :vskip 10
      :text USER2PRETRIG         :label "PreTrigger (card)"  :w 8 :valuex 24c

   :container end

if {$UserGUIVersion == 2} {     ;#   MWD firmware

   :panel P1L :layout T :padx 5 :pady 5
     :layout H; :row-gap 10; :col-gap 10

     set UserReg {}
     for {set reg 16} {$reg <= 31} {incr reg} {lappend UserReg $reg}

      :text MWDR16  :label "MWDR16"  :w 12 :valuex 10c
      :text MWDR17  :label "MWDR17"  :w 12 :valuex 10c
      :text MWDR18  :label "MWDR18"  :w 12 :valuex 10c
      :text MWDR19  :label "MWDR19"  :w 12 :valuex 10c
      :next-row T

      :text MWDR20  :label "MWDR20"  :w 12 :valuex 10c
      :text MWDR21  :label "MWDR21"  :w 12 :valuex 10c
      :text MWDR22  :label "MWDR22"  :w 12 :valuex 10c
      :text MWDR23  :label "MWDR23"  :w 12 :valuex 10c
      :next-row T

      :text MWDR25  :label "MWDR25"  :w 12 :valuex 10c
      :text MWDR26  :label "MWDR26"  :w 12 :valuex 10c
      :text MWDR27  :label "MWDR27"  :w 12 :valuex 10c
      :text MWDR28  :label "MWDR28"  :w 12 :valuex 10c
      :next-row T

      :text MWDR30  :label "MWDR30"  :w 12 :valuex 10c
      :hskip 25
      :message USER2ACT :label "MWD parameter changes are now activated on GO" :bold T
}

   :panel P1 :layout B :padx 5 :pady 5   :after FOOTER
     :layout H; :row-gap 10; :col-gap 10

      :text USER21          :label "User Register 1" :inactive T :w 12
      :choice-stack USER210 :label channel :value $user10 :strings $Menu1 :valuex 12c
      :choice-stack USER211 :label "register address" :value $user11 :strings $Menu2 :valuex 16c
      :text USER212         :label "register value" :w 8 :value $user12 :valuex 12c
      :button USER213       :label "Write"
      :next-row T

      :text USER22          :label "User Register 2" :inactive T :w 12
      :choice-stack USER220 :label channel :value $user20 :strings $Menu1 :valuex 12c
      :choice-stack USER221 :label "register address" :value $user21 :strings $Menu2 :valuex 16c
      :text USER222         :label "register value" :w 8 :value $user22 :valuex 12c
      :button USER223       :label "Read"
      :next-row T

   :panel BUTTONS :layout B :padx 5 :pady 5  :after FOOTER
     :layout H; :row-gap 10; :col-gap 10

      :button USERDONE  :label "Done" :command "frame-quit [self frame-name].user"
      :button USERUPDTE :label "Update" :command update_UserGUI
      :hskip 50
      :button PAUSE   :label "Pause"   :command do_pause
      :button RESUME  :label "Resume"  :command do_resume
      :hskip 50
      :message WARNING     :label  "Warning: Access to firmware parameters is only reliable when not GOING" :bold T

  :show T

   update_UserGUI
}

proc Read_MWD_Registers {} {
    global boardID chan
    global RegMask
    global UserReg
    global UserGUIVersion

    global MWDr0
    foreach reg $UserReg {global MWDr$reg}

    set registers {}    ;#  build list of registers

    set reg 0
    lappend registers  [expr ($chan << 28) + ($reg << 16)]

    foreach reg $UserReg {lappend registers  [expr ($chan << 28) + ($reg << 16)]}

    set reg 0x20    ;#  Adding CFD Threshold
    lappend registers  [expr ($chan << 28) + ($reg << 16)]

    set reg 0x31    ;#  Adding  PreTrigger Position
    lappend registers  [expr ($chan << 28) + ($reg << 16)]

    if {$UserGUIVersion == 2} {     ;#   MWD firmware
       foreach reg [list 0x3a 0x3b 0x3c 0x3d] {
           lappend registers  [expr ($chan << 28) + ($reg << 16)]
       }
    }

    set values [VHSADCClient__ReadUserRegs $boardID $registers]   ;# get all the values

    set index 0

    set reg 0
    set rc [lindex $values $index]; incr index
    set MWDr$reg [expr $rc & $RegMask($reg)]

    foreach reg $UserReg {
        set rc [lindex $values $index]; incr index
        set MWDr$reg [expr $rc & $RegMask($reg)]
    }

    foreach reg [list 0x20] {
        global Reg$reg
        set rc [lindex $values $index]; incr index
        set Reg$reg [expr $rc & 0x0000ffff]
    }

    foreach reg [list 0x31] {
        global Reg$reg
        set rc [lindex $values $index]; incr index
        set Reg$reg [expr $rc & 0x0000ffff]
    }

    if {$UserGUIVersion == 2} {     ;#   MWD firmware

        foreach reg [list 0x3a 0x3b 0x3c 0x3d] {
           global Reg$reg
           set rc [lindex $values $index]; incr index
           set Reg$reg [expr $rc & 0x0000ffff]
        }

    }

}

proc Set_MWD_Registers {} {
    global boardID chanlist NumChan
    global UserGUIVersion

    if {$UserGUIVersion == 2} {     ;#   MWD firmware
        update_mwd_parameters
        apply_mwd_parameters
    }

    update_UserGUI   ;# if needed

}

proc update_UserGUI {} {
    global boardID WriteIF ReadIF
    global chan
    global UserGUIVersion
    global UserReg

    for {set reg 16} {$reg <= 31} {incr reg} {global MWDr$reg}
    global Reg0x20 Reg0x31

    :modify [self frame-name].user :busy T

    :modify USER20  :inactive F :value $boardID :inactive T

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID $WriteIF]  ;# user register
    :modify USER21  :inactive F :value [format 0x%08x $rc] :inactive T

    set rc [VHSADCClient__Vhsadac16ReadFpgaReg $boardID $ReadIF]  ;# user register
    :modify USER22  :inactive F :value [format 0x%08x $rc] :inactive T

    set reg 0                                               ;# Control Word
        set-footer [self frame-name].user "Reading register $reg"
        set rc [VHSADCClient__ReadUserReg $chan $reg]
        :modify USER2CW  :value [expr $rc & 0x0000ffff]

    set reg 1                                               ;# Status Word
        set-footer [self frame-name].user "Reading register $reg"
        set rc [VHSADCClient__ReadUserReg $chan $reg]
        :modify USER2SW  :inactive F :value [expr $rc & 0x000001ff] :inactive T

    set-footer [self frame-name].user "Reading MWD registers"
    Read_MWD_Registers
    clear-footer [self frame-name].user

    if {$UserGUIVersion == 2} {     ;#   MWD firmware

        foreach reg [list 16 17 18 19 20 21 22 25 26 27] {
            :modify MWDR$reg :value [format "%d (0x%04x)" [set MWDr$reg] [set MWDr$reg]]
        }

        :modify MWDR23 :value [format 0x%08x [expr ($MWDr24 << 16) + $MWDr23]]
        :modify MWDR28 :value [format 0x%08x [expr ($MWDr29 << 16) + $MWDr28]]
        :modify MWDR30 :value [format 0x%08x [expr ($MWDr31 << 16) + $MWDr30]]

    }

    if {$UserGUIVersion == 1} { 
        foreach reg $UserReg {
            :modify MWDR$reg :value [set MWDr$reg]
        }
    }

    :modify USER2CFD  :value [expr $Reg0x20 & 0x0000ffff]
    :modify USER2PRETRIG  :value [expr $Reg0x31 & 0x0000ffff]

    if {$UserGUIVersion == 2} {     ;#   MWD firmware
        update_mwd_parameters
    }

    :modify [self frame-name].user :busy F
}

proc click_UserGUI {id vl index} {
    global boardID boardlist chan chanlist NumChan
    global allmodules allchans

#insert-debug "click_UserGUI with $id $vl $index"

    set chan_SAV $chan
    set boardID_SAV $boardID

   switch $id {
     10 -
     11 -
     12 -
     13 -
     20 -
     21 -
     22 -
     23 {
          :modify [self frame-name].user :busy T
          do_click_UserGUI $chan $id $vl $index
          :modify [self frame-name].user :busy F
          return
     }
   }

    set state [inform sys inquire readoutstate]
    if {$state != 0} {
        midas-warning "You can only do this reliably when STOPped"
        return
    }

    :modify [self frame-name].user :busy T

    if {$allmodules == 0} {

        if {$allchans == 0} {
            do_click_UserGUI $chan $id $vl $index
        } else {
            foreach ch $chanlist {
                 if {$ch >= $NumChan($boardID)} {break}
                 set-footer [self frame-name].user "Setting board $boardID; channel $ch"
                 do_click_UserGUI $ch $id $vl $index
            }
        }
    } else {
        foreach boardID $boardlist {

            if {$allchans == 0} {
                set-footer [self frame-name].user "Setting board $boardID; channel $chan"
                do_click_UserGUI $chan $id $vl $index
            } else {
                foreach ch $chanlist {
                    if {$ch >= $NumChan($boardID)} {break}
                    set-footer [self frame-name].user "Setting board $boardID; channel $ch"
                    do_click_UserGUI $ch $id $vl $index
                }
            }
        }
    }

    set boardID $boardID_SAV
    set chan $chan_SAV

    :modify [self frame-name].user :busy F
    clear-footer [self frame-name].user

    if {$allchans != 0} {click-select-chan $chan}

}

proc do_click_UserGUI {chan id vl index} {
    global UserGUIVersion
    global boardID WriteIF ReadIF
    global user10 user11 user12
    global user20 user21 user22
    global RegMask
    for {set reg 16} {$reg <= 31} {incr reg} {global MWDr$reg}

#insert-debug "do_click_UserGUI with $boardID $chan $id $vl $index"

    switch $id {
        10 {set user10 $index}
        11 {set user11 $index}
        12 {set user12 $vl}
        13 {                          ;# click on button USER213   (Write User Register 1)
            VHSADCClient__WriteUserReg $user10 $user11 $user12
        }
        20 {set user20 $index}
        21 {set user21 $index}
        22 {set user22 $vl}
        23 {                          ;# click on button USER223   (Read User Register 2)
            set rc [VHSADCClient__ReadUserReg $user20 $user21]
            :modify USER22  :inactive F :value [format 0x%08x $rc] :inactive T
            if {[expr ($rc >> 28) & 15] == $user20 && [expr ($rc >> 16) & 0x0fff] == $user21} {
                :modify USER222 :value [format 0x%04x [expr $rc & 0x0000ffff]]
            } else {
                :modify USER222 :value "******"
            }
        }
        R16 -
        R17 -
        R18 -
        R19 -
        R20 -
        R21 -
        R22 -
        R25 -
        R26 -
        R27 -
        R28 -
        R29 -
        R30 -
        R31 {
            set reg [string range $id 1 end]
            VHSADCClient__WriteUserReg $chan $reg $vl
            set rc [VHSADCClient__ReadUserReg $chan $reg]
            set MWDr$reg [expr $rc & $RegMask($reg)]
            if {$UserGUIVersion == 2} {     ;#   MWD firmware
                :modify MWDR$reg :value [format "%d (0x%04x)" [set MWDr$reg] [set MWDr$reg]]
            }
            if {$UserGUIVersion == 1} { 
                :modify MWDR$reg :value [set MWDr$reg]
            }
        }
        R23 {
            VHSADCClient__WriteUserReg $chan 23 [expr $vl & 0x0000ffff]
            VHSADCClient__WriteUserReg $chan 24 [expr ($vl >> 16) & 0x000000ff]
            set MWDr23 [expr [VHSADCClient__ReadUserReg $chan 23] & $RegMask(23)]
            set MWDr24 [expr [VHSADCClient__ReadUserReg $chan 24] & $RegMask(24)]
            :modify MWDR23 :value [format 0x%08x [expr ($MWDr24 << 16) + $MWDr23]]
        }
        CW {
            VHSADCClient__WriteUserReg $chan 0 [expr $vl & 0x0000ffff]
            set rc [VHSADCClient__ReadUserReg $chan 0]
            :modify USER2CW :value [format 0x%04x [expr $rc & 0x0000ffff]]
        }
        CFD {
            VHSADCClient__WriteUserReg $chan 0x20 [expr $vl & 0x0000ffff]
            set rc [VHSADCClient__ReadUserReg $chan 0x20]
            :modify USER2CFD :value [expr $rc & 0x0000ffff]
        }
        PRETRIG {
            set reg 0x31
            VHSADCClient__WriteUserReg 0 $reg [expr $vl & 0x0000ffff]
            set rc [VHSADCClient__ReadUserReg 0 $reg]
            :modify USER2PRETRIG :value [expr $rc & 0x0000ffff]
            set Activate 1
        }
    }

    if {$UserGUIVersion == 2} {     ;#   MWD firmware
        update_mwd_parameters
    }
}

proc click-user2cfd {v} {

    click_UserGUI CFD $v 0

}

proc click-user2pretrig {v} {

    click_UserGUI PRETRIG $v 0

}

proc click-clock_frequency  {v} {
    global UserGUIVersion
    global clock_frequency ADCClockFreq

    if {$ADCClockFreq != "External"} {
        midas-warning "For internal clocks you should set the frequency when loading the FPGA fimrware"
        return
    }

    set clock_frequency $v
    if {$UserGUIVersion == 2} {     ;#   MWD firmware
        apply_mwd_parameters
        update_mwd_parameters
    }
}

proc click-decimation_factor  {v} {
    global decimation_factor
    global MWDp9

    set decimation_factor $v
    apply_mwd_parameters
    click_UserGUI R25 $MWDp9 0
}

proc click-shaping_time  {v} {
    global shaping_time
    global MWDp0 MWDp7

    set shaping_time $v
    apply_mwd_parameters
    click_UserGUI R16 $MWDp0 0
    click_UserGUI R23 $MWDp7 0
}

proc click-trapezoid_width   {v} {
    global trapezoid_width
    global MWDp1

    set trapezoid_width $v
    apply_mwd_parameters
    click_UserGUI R17 $MWDp1 0
}

proc click-peak_separation  {v} {
    global peak_separation
    global MWDp3

    set peak_separation $v
    apply_mwd_parameters
    click_UserGUI R19 $MWDp3 0
}

proc click-decay_time_constant  {v} {
    global decay_time_constant
    global MWDp7

    set decay_time_constant $v
    apply_mwd_parameters
    click_UserGUI R23 $MWDp7 0
}

proc click-peak_sample_time   {v} {
    global peak_sample_time
    global MWDp2

    set peak_sample_time $v
    apply_mwd_parameters
    click_UserGUI R18 $MWDp2 0
}

proc click-baseline_update_time   {v} {
    global baseline_update_time
    global MWDp5

    set baseline_update_time $v
    apply_mwd_parameters
    click_UserGUI R21 $MWDp5 0
}

proc click-baseline_average {v} {
    global baseline_average
    global MWDp4

    set baseline_average $v
    apply_mwd_parameters
    click_UserGUI R20 $MWDp4 0
}

proc click-baseline_threshold {v} {
    global baseline_threshold
    global MWDp6

    set baseline_threshold $v
    apply_mwd_parameters
    click_UserGUI R22 $MWDp6 0
}

proc click-mode {v} {
    global mode
    global MWDp10

    set mode $v
    apply_mwd_parameters
    click_UserGUI R26 $MWDp10 0
}

proc apply_mwd_parameters {} {
    global clock_frequency decimation_factor shaping_time  trapezoid_width
    global peak_separation decay_time_constant peak_sample_time
    global baseline_average  baseline_update_time baseline_threshold mode
    global ADCClockFreq

    for {set i 0} {$i < 11} {incr i} { global MWDp$i }

    if {$ADCClockFreq != "External"} {
        if {$clock_frequency != $ADCClockFreq} {set clock_frequency $ADCClockFreq;  update_mwd_parameters}
    }

    set clock_width [expr 1000000000 / ( [set clock_frequency] * 1000000 ) ]
    set ddt [expr $clock_width * 0.001 * $decimation_factor ]
    set dt [expr $clock_width * 0.001 ]
    set Mbydt [expr $shaping_time / $ddt]

    set MWDp0 [expr round(256 - $Mbydt +1)]                             ;# R16
    set MWDp1 [expr 256 - round($trapezoid_width / $ddt ) -1]           ;# R17
    set MWDp2 [expr int($peak_sample_time / $dt)]                       ;# R18
    set MWDp3 [expr int( $peak_separation / $dt ) ]                     ;# R19
    set MWDp4 $baseline_average                                         ;# R20
    set MWDp5 [expr round($baseline_update_time / $ddt)]                ;# R21
    set MWDp6 $baseline_threshold                                       ;# R22

    set MWDp7 [expr round(16777216 * (1 - exp([expr - $shaping_time / $decay_time_constant])) / $Mbydt)]    ;# R23

    set MWDp9 [expr 32 - $decimation_factor +1]                         ;# R25
    set MWDp10 $mode                                                    ;# R26

}

proc update_mwd_parameters {} {
    global clock_frequency decimation_factor shaping_time  trapezoid_width
    global peak_separation decay_time_constant peak_sample_time 
    global baseline_average baseline_update_time baseline_threshold mode
    global ADCClockFreq

    for {set reg 16} {$reg <= 31} {incr reg} {global MWDr$reg}

    if {$ADCClockFreq != "External"} {
        if {$clock_frequency != $ADCClockFreq} {set clock_frequency $ADCClockFreq;  apply_mwd_parameters}
    }

    set clock_width [expr 1000000000 / ( [set clock_frequency] * 1000000 ) ]
    set decimation_factor [expr 33 - $MWDr25]
    set ddt [expr $clock_width * 0.001 * $decimation_factor ]
    set dt [expr $clock_width * 0.001 ]
    set shaping_time [expr (257 - $MWDr16) * $ddt]
    set Mbydt [expr $shaping_time / $ddt]

#    set shaping_time [expr 257 - $MWDr16]
    set trapezoid_width [expr (255 - $MWDr17) * $ddt]
    set peak_sample_time [expr $MWDr18 * $dt]
    set peak_separation [expr $MWDr19 * $dt]
    set baseline_average $MWDr20
    set baseline_update_time [expr round($MWDr21 * $ddt)]
    set baseline_threshold $MWDr22
    set p1 [expr ($MWDr23*$Mbydt)/16777216]
    set p2 [expr 1/(1-$p1)]
    set decay_time_constant [expr $shaping_time/log($p2)]
#    set decimation_factor [expr 33 - $MWDr25]
    set mode $MWDr26

    :modify CLOCK_FREQUENCY      :value [string range $clock_frequency 0 2]
    :modify DECIMATION_FACTOR    :value [string range $decimation_factor 0 5]
    :modify SHAPING_TIME         :value [string range $shaping_time 0 5]
    :modify TRAPEZOID_WIDTH      :value [string range $trapezoid_width 0 5]
    :modify PEAK_SEPARATION      :value [string range $peak_separation 0 5]
    :modify DECAY_TIME_CONSTANT  :value [string range $decay_time_constant 0 5]
    :modify PEAK_SAMPLE_TIME     :value [string range $peak_sample_time 0 5]
    :modify BASELINE_UPDATE_TIME :value [string range $baseline_update_time 0 5]
    :modify BASELINE_AVERAGE     :value [string range $baseline_average 0 5]
    :modify BASELINE_THRESHOLD   :value [string range $baseline_threshold 0 5]
    :modify MODE                 :value [string range $mode 0 5]

}
